/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "__generics.h"
#include "vtype.h"

#ifndef LIBCDSB_MAP_H
#define LIBCDSB_MAP_H

extern void map_init(vtype_map* x, vtype key_type);

#define map_pop(x, s, key)        _LIBCDSB_Generic (libcdsb_map, find, key)(x, s, key, 1)
#define map_get(x, s, key)        _LIBCDSB_Generic (libcdsb_map, find, key)(x, s, key, 0)
#define map_update(x, key, value) _LIBCDSB_Generic2(libcdsb_map, update, key, value)(x, key, value)
#define map_remove(x, key)        map_pop(0, x, key)

extern _Bool libcdsb_map_find_pointer(vtype_value* x, vtype_map* s, const         void* key, _Bool cut);
extern _Bool libcdsb_map_find_cstring(vtype_value* x, vtype_map* s, const         char* key, _Bool cut);
extern _Bool libcdsb_map_find_string (vtype_value* x, vtype_map* s, const vtype_string* key, _Bool cut);
extern _Bool libcdsb_map_find_array  (vtype_value* x, vtype_map* s, const  vtype_array* key, _Bool cut);
extern _Bool libcdsb_map_find_list   (vtype_value* x, vtype_map* s, const   vtype_list* key, _Bool cut);
extern _Bool libcdsb_map_find_map    (vtype_value* x, vtype_map* s, const    vtype_map* key, _Bool cut);
extern _Bool libcdsb_map_find_vset   (vtype_value* x, vtype_map* s, const    vtype_set* key, _Bool cut);
extern _Bool libcdsb_map_find_boolean(vtype_value* x, vtype_map* s,          vtype_bool key, _Bool cut);
extern _Bool libcdsb_map_find_int8   (vtype_value* x, vtype_map* s,          vtype_int8 key, _Bool cut);
extern _Bool libcdsb_map_find_int16  (vtype_value* x, vtype_map* s,         vtype_int16 key, _Bool cut);
extern _Bool libcdsb_map_find_int32  (vtype_value* x, vtype_map* s,         vtype_int32 key, _Bool cut);
extern _Bool libcdsb_map_find_int64  (vtype_value* x, vtype_map* s,         vtype_int64 key, _Bool cut);
extern _Bool libcdsb_map_find_uint8  (vtype_value* x, vtype_map* s,         vtype_uint8 key, _Bool cut);
extern _Bool libcdsb_map_find_uint16 (vtype_value* x, vtype_map* s,        vtype_uint16 key, _Bool cut);
extern _Bool libcdsb_map_find_uint32 (vtype_value* x, vtype_map* s,        vtype_uint32 key, _Bool cut);
extern _Bool libcdsb_map_find_uint64 (vtype_value* x, vtype_map* s,        vtype_uint64 key, _Bool cut);
extern _Bool libcdsb_map_find_float  (vtype_value* x, vtype_map* s,         vtype_float key, _Bool cut);
extern _Bool libcdsb_map_find_double (vtype_value* x, vtype_map* s,        vtype_double key, _Bool cut);
extern _Bool libcdsb_map_find_ldouble(vtype_value* x, vtype_map* s,       vtype_ldouble key, _Bool cut);

extern _Bool libcdsb_map_update_pointer_pointer(vtype_map* x, const void* key, const         void* value);
extern _Bool libcdsb_map_update_pointer_cstring(vtype_map* x, const void* key, const         char* value);
extern _Bool libcdsb_map_update_pointer_string (vtype_map* x, const void* key, const vtype_string* value);
extern _Bool libcdsb_map_update_pointer_array  (vtype_map* x, const void* key, const  vtype_array* value);
extern _Bool libcdsb_map_update_pointer_list   (vtype_map* x, const void* key, const   vtype_list* value);
extern _Bool libcdsb_map_update_pointer_map    (vtype_map* x, const void* key, const    vtype_map* value);
extern _Bool libcdsb_map_update_pointer_vset   (vtype_map* x, const void* key, const    vtype_set* value);
extern _Bool libcdsb_map_update_pointer_boolean(vtype_map* x, const void* key,          vtype_bool value);
extern _Bool libcdsb_map_update_pointer_int8   (vtype_map* x, const void* key,          vtype_int8 value);
extern _Bool libcdsb_map_update_pointer_int16  (vtype_map* x, const void* key,         vtype_int16 value);
extern _Bool libcdsb_map_update_pointer_int32  (vtype_map* x, const void* key,         vtype_int32 value);
extern _Bool libcdsb_map_update_pointer_int64  (vtype_map* x, const void* key,         vtype_int64 value);
extern _Bool libcdsb_map_update_pointer_uint8  (vtype_map* x, const void* key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_pointer_uint16 (vtype_map* x, const void* key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_pointer_uint32 (vtype_map* x, const void* key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_pointer_uint64 (vtype_map* x, const void* key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_pointer_float  (vtype_map* x, const void* key,         vtype_float value);
extern _Bool libcdsb_map_update_pointer_double (vtype_map* x, const void* key,        vtype_double value);
extern _Bool libcdsb_map_update_pointer_ldouble(vtype_map* x, const void* key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_string_pointer(vtype_map* x, const vtype_string* key, const         void* value);
extern _Bool libcdsb_map_update_string_cstring(vtype_map* x, const vtype_string* key, const         char* value);
extern _Bool libcdsb_map_update_string_string (vtype_map* x, const vtype_string* key, const vtype_string* value);
extern _Bool libcdsb_map_update_string_array  (vtype_map* x, const vtype_string* key, const  vtype_array* value);
extern _Bool libcdsb_map_update_string_list   (vtype_map* x, const vtype_string* key, const   vtype_list* value);
extern _Bool libcdsb_map_update_string_map    (vtype_map* x, const vtype_string* key, const    vtype_map* value);
extern _Bool libcdsb_map_update_string_vset   (vtype_map* x, const vtype_string* key, const    vtype_set* value);
extern _Bool libcdsb_map_update_string_boolean(vtype_map* x, const vtype_string* key,          vtype_bool value);
extern _Bool libcdsb_map_update_string_int8   (vtype_map* x, const vtype_string* key,          vtype_int8 value);
extern _Bool libcdsb_map_update_string_int16  (vtype_map* x, const vtype_string* key,         vtype_int16 value);
extern _Bool libcdsb_map_update_string_int32  (vtype_map* x, const vtype_string* key,         vtype_int32 value);
extern _Bool libcdsb_map_update_string_int64  (vtype_map* x, const vtype_string* key,         vtype_int64 value);
extern _Bool libcdsb_map_update_string_uint8  (vtype_map* x, const vtype_string* key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_string_uint16 (vtype_map* x, const vtype_string* key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_string_uint32 (vtype_map* x, const vtype_string* key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_string_uint64 (vtype_map* x, const vtype_string* key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_string_float  (vtype_map* x, const vtype_string* key,         vtype_float value);
extern _Bool libcdsb_map_update_string_double (vtype_map* x, const vtype_string* key,        vtype_double value);
extern _Bool libcdsb_map_update_string_ldouble(vtype_map* x, const vtype_string* key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_array_pointer(vtype_map* x, const vtype_array* key, const         void* value);
extern _Bool libcdsb_map_update_array_cstring(vtype_map* x, const vtype_array* key, const         char* value);
extern _Bool libcdsb_map_update_array_string (vtype_map* x, const vtype_array* key, const vtype_string* value);
extern _Bool libcdsb_map_update_array_array  (vtype_map* x, const vtype_array* key, const  vtype_array* value);
extern _Bool libcdsb_map_update_array_list   (vtype_map* x, const vtype_array* key, const   vtype_list* value);
extern _Bool libcdsb_map_update_array_map    (vtype_map* x, const vtype_array* key, const    vtype_map* value);
extern _Bool libcdsb_map_update_array_vset   (vtype_map* x, const vtype_array* key, const    vtype_set* value);
extern _Bool libcdsb_map_update_array_boolean(vtype_map* x, const vtype_array* key,          vtype_bool value);
extern _Bool libcdsb_map_update_array_int8   (vtype_map* x, const vtype_array* key,          vtype_int8 value);
extern _Bool libcdsb_map_update_array_int16  (vtype_map* x, const vtype_array* key,         vtype_int16 value);
extern _Bool libcdsb_map_update_array_int32  (vtype_map* x, const vtype_array* key,         vtype_int32 value);
extern _Bool libcdsb_map_update_array_int64  (vtype_map* x, const vtype_array* key,         vtype_int64 value);
extern _Bool libcdsb_map_update_array_uint8  (vtype_map* x, const vtype_array* key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_array_uint16 (vtype_map* x, const vtype_array* key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_array_uint32 (vtype_map* x, const vtype_array* key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_array_uint64 (vtype_map* x, const vtype_array* key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_array_float  (vtype_map* x, const vtype_array* key,         vtype_float value);
extern _Bool libcdsb_map_update_array_double (vtype_map* x, const vtype_array* key,        vtype_double value);
extern _Bool libcdsb_map_update_array_ldouble(vtype_map* x, const vtype_array* key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_list_pointer(vtype_map* x, const vtype_list* key, const         void* value);
extern _Bool libcdsb_map_update_list_cstring(vtype_map* x, const vtype_list* key, const         char* value);
extern _Bool libcdsb_map_update_list_string (vtype_map* x, const vtype_list* key, const vtype_string* value);
extern _Bool libcdsb_map_update_list_array  (vtype_map* x, const vtype_list* key, const  vtype_array* value);
extern _Bool libcdsb_map_update_list_list   (vtype_map* x, const vtype_list* key, const   vtype_list* value);
extern _Bool libcdsb_map_update_list_map    (vtype_map* x, const vtype_list* key, const    vtype_map* value);
extern _Bool libcdsb_map_update_list_vset   (vtype_map* x, const vtype_list* key, const    vtype_set* value);
extern _Bool libcdsb_map_update_list_boolean(vtype_map* x, const vtype_list* key,          vtype_bool value);
extern _Bool libcdsb_map_update_list_int8   (vtype_map* x, const vtype_list* key,          vtype_int8 value);
extern _Bool libcdsb_map_update_list_int16  (vtype_map* x, const vtype_list* key,         vtype_int16 value);
extern _Bool libcdsb_map_update_list_int32  (vtype_map* x, const vtype_list* key,         vtype_int32 value);
extern _Bool libcdsb_map_update_list_int64  (vtype_map* x, const vtype_list* key,         vtype_int64 value);
extern _Bool libcdsb_map_update_list_uint8  (vtype_map* x, const vtype_list* key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_list_uint16 (vtype_map* x, const vtype_list* key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_list_uint32 (vtype_map* x, const vtype_list* key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_list_uint64 (vtype_map* x, const vtype_list* key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_list_float  (vtype_map* x, const vtype_list* key,         vtype_float value);
extern _Bool libcdsb_map_update_list_double (vtype_map* x, const vtype_list* key,        vtype_double value);
extern _Bool libcdsb_map_update_list_ldouble(vtype_map* x, const vtype_list* key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_map_pointer(vtype_map* x, const vtype_map* key, const         void* value);
extern _Bool libcdsb_map_update_map_cstring(vtype_map* x, const vtype_map* key, const         char* value);
extern _Bool libcdsb_map_update_map_string (vtype_map* x, const vtype_map* key, const vtype_string* value);
extern _Bool libcdsb_map_update_map_array  (vtype_map* x, const vtype_map* key, const  vtype_array* value);
extern _Bool libcdsb_map_update_map_list   (vtype_map* x, const vtype_map* key, const   vtype_list* value);
extern _Bool libcdsb_map_update_map_map    (vtype_map* x, const vtype_map* key, const    vtype_map* value);
extern _Bool libcdsb_map_update_map_vset   (vtype_map* x, const vtype_map* key, const    vtype_set* value);
extern _Bool libcdsb_map_update_map_boolean(vtype_map* x, const vtype_map* key,          vtype_bool value);
extern _Bool libcdsb_map_update_map_int8   (vtype_map* x, const vtype_map* key,          vtype_int8 value);
extern _Bool libcdsb_map_update_map_int16  (vtype_map* x, const vtype_map* key,         vtype_int16 value);
extern _Bool libcdsb_map_update_map_int32  (vtype_map* x, const vtype_map* key,         vtype_int32 value);
extern _Bool libcdsb_map_update_map_int64  (vtype_map* x, const vtype_map* key,         vtype_int64 value);
extern _Bool libcdsb_map_update_map_uint8  (vtype_map* x, const vtype_map* key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_map_uint16 (vtype_map* x, const vtype_map* key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_map_uint32 (vtype_map* x, const vtype_map* key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_map_uint64 (vtype_map* x, const vtype_map* key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_map_float  (vtype_map* x, const vtype_map* key,         vtype_float value);
extern _Bool libcdsb_map_update_map_double (vtype_map* x, const vtype_map* key,        vtype_double value);
extern _Bool libcdsb_map_update_map_ldouble(vtype_map* x, const vtype_map* key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_vset_pointer(vtype_map* x, const vtype_set* key, const         void* value);
extern _Bool libcdsb_map_update_vset_cstring(vtype_map* x, const vtype_set* key, const         char* value);
extern _Bool libcdsb_map_update_vset_string (vtype_map* x, const vtype_set* key, const vtype_string* value);
extern _Bool libcdsb_map_update_vset_array  (vtype_map* x, const vtype_set* key, const  vtype_array* value);
extern _Bool libcdsb_map_update_vset_list   (vtype_map* x, const vtype_set* key, const   vtype_list* value);
extern _Bool libcdsb_map_update_vset_map    (vtype_map* x, const vtype_set* key, const    vtype_map* value);
extern _Bool libcdsb_map_update_vset_vset   (vtype_map* x, const vtype_set* key, const    vtype_set* value);
extern _Bool libcdsb_map_update_vset_boolean(vtype_map* x, const vtype_set* key,          vtype_bool value);
extern _Bool libcdsb_map_update_vset_int8   (vtype_map* x, const vtype_set* key,          vtype_int8 value);
extern _Bool libcdsb_map_update_vset_int16  (vtype_map* x, const vtype_set* key,         vtype_int16 value);
extern _Bool libcdsb_map_update_vset_int32  (vtype_map* x, const vtype_set* key,         vtype_int32 value);
extern _Bool libcdsb_map_update_vset_int64  (vtype_map* x, const vtype_set* key,         vtype_int64 value);
extern _Bool libcdsb_map_update_vset_uint8  (vtype_map* x, const vtype_set* key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_vset_uint16 (vtype_map* x, const vtype_set* key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_vset_uint32 (vtype_map* x, const vtype_set* key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_vset_uint64 (vtype_map* x, const vtype_set* key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_vset_float  (vtype_map* x, const vtype_set* key,         vtype_float value);
extern _Bool libcdsb_map_update_vset_double (vtype_map* x, const vtype_set* key,        vtype_double value);
extern _Bool libcdsb_map_update_vset_ldouble(vtype_map* x, const vtype_set* key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_cstring_pointer(vtype_map* x, const char* key, const         void* value);
extern _Bool libcdsb_map_update_cstring_cstring(vtype_map* x, const char* key, const         char* value);
extern _Bool libcdsb_map_update_cstring_string (vtype_map* x, const char* key, const vtype_string* value);
extern _Bool libcdsb_map_update_cstring_array  (vtype_map* x, const char* key, const  vtype_array* value);
extern _Bool libcdsb_map_update_cstring_list   (vtype_map* x, const char* key, const   vtype_list* value);
extern _Bool libcdsb_map_update_cstring_map    (vtype_map* x, const char* key, const    vtype_map* value);
extern _Bool libcdsb_map_update_cstring_vset   (vtype_map* x, const char* key, const    vtype_set* value);
extern _Bool libcdsb_map_update_cstring_boolean(vtype_map* x, const char* key,          vtype_bool value);
extern _Bool libcdsb_map_update_cstring_int8   (vtype_map* x, const char* key,          vtype_int8 value);
extern _Bool libcdsb_map_update_cstring_int16  (vtype_map* x, const char* key,         vtype_int16 value);
extern _Bool libcdsb_map_update_cstring_int32  (vtype_map* x, const char* key,         vtype_int32 value);
extern _Bool libcdsb_map_update_cstring_int64  (vtype_map* x, const char* key,         vtype_int64 value);
extern _Bool libcdsb_map_update_cstring_uint8  (vtype_map* x, const char* key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_cstring_uint16 (vtype_map* x, const char* key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_cstring_uint32 (vtype_map* x, const char* key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_cstring_uint64 (vtype_map* x, const char* key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_cstring_float  (vtype_map* x, const char* key,         vtype_float value);
extern _Bool libcdsb_map_update_cstring_double (vtype_map* x, const char* key,        vtype_double value);
extern _Bool libcdsb_map_update_cstring_ldouble(vtype_map* x, const char* key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_boolean_pointer(vtype_map* x, vtype_bool key, const         void* value);
extern _Bool libcdsb_map_update_boolean_cstring(vtype_map* x, vtype_bool key, const         char* value);
extern _Bool libcdsb_map_update_boolean_string (vtype_map* x, vtype_bool key, const vtype_string* value);
extern _Bool libcdsb_map_update_boolean_array  (vtype_map* x, vtype_bool key, const  vtype_array* value);
extern _Bool libcdsb_map_update_boolean_list   (vtype_map* x, vtype_bool key, const   vtype_list* value);
extern _Bool libcdsb_map_update_boolean_map    (vtype_map* x, vtype_bool key, const    vtype_map* value);
extern _Bool libcdsb_map_update_boolean_vset   (vtype_map* x, vtype_bool key, const    vtype_set* value);
extern _Bool libcdsb_map_update_boolean_boolean(vtype_map* x, vtype_bool key,          vtype_bool value);
extern _Bool libcdsb_map_update_boolean_int8   (vtype_map* x, vtype_bool key,          vtype_int8 value);
extern _Bool libcdsb_map_update_boolean_int16  (vtype_map* x, vtype_bool key,         vtype_int16 value);
extern _Bool libcdsb_map_update_boolean_int32  (vtype_map* x, vtype_bool key,         vtype_int32 value);
extern _Bool libcdsb_map_update_boolean_int64  (vtype_map* x, vtype_bool key,         vtype_int64 value);
extern _Bool libcdsb_map_update_boolean_uint8  (vtype_map* x, vtype_bool key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_boolean_uint16 (vtype_map* x, vtype_bool key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_boolean_uint32 (vtype_map* x, vtype_bool key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_boolean_uint64 (vtype_map* x, vtype_bool key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_boolean_float  (vtype_map* x, vtype_bool key,         vtype_float value);
extern _Bool libcdsb_map_update_boolean_double (vtype_map* x, vtype_bool key,        vtype_double value);
extern _Bool libcdsb_map_update_boolean_ldouble(vtype_map* x, vtype_bool key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_uint8_pointer(vtype_map* x, vtype_uint8 key, const         void* value);
extern _Bool libcdsb_map_update_uint8_cstring(vtype_map* x, vtype_uint8 key, const         char* value);
extern _Bool libcdsb_map_update_uint8_string (vtype_map* x, vtype_uint8 key, const vtype_string* value);
extern _Bool libcdsb_map_update_uint8_array  (vtype_map* x, vtype_uint8 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_uint8_list   (vtype_map* x, vtype_uint8 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_uint8_map    (vtype_map* x, vtype_uint8 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_uint8_vset   (vtype_map* x, vtype_uint8 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_uint8_boolean(vtype_map* x, vtype_uint8 key,          vtype_bool value);
extern _Bool libcdsb_map_update_uint8_int8   (vtype_map* x, vtype_uint8 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_uint8_int16  (vtype_map* x, vtype_uint8 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_uint8_int32  (vtype_map* x, vtype_uint8 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_uint8_int64  (vtype_map* x, vtype_uint8 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_uint8_uint8  (vtype_map* x, vtype_uint8 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_uint8_uint16 (vtype_map* x, vtype_uint8 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_uint8_uint32 (vtype_map* x, vtype_uint8 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_uint8_uint64 (vtype_map* x, vtype_uint8 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_uint8_float  (vtype_map* x, vtype_uint8 key,         vtype_float value);
extern _Bool libcdsb_map_update_uint8_double (vtype_map* x, vtype_uint8 key,        vtype_double value);
extern _Bool libcdsb_map_update_uint8_ldouble(vtype_map* x, vtype_uint8 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_uint16_pointer(vtype_map* x, vtype_uint16 key, const         void* value);
extern _Bool libcdsb_map_update_uint16_cstring(vtype_map* x, vtype_uint16 key, const         char* value);
extern _Bool libcdsb_map_update_uint16_string (vtype_map* x, vtype_uint16 key, const vtype_string* value);
extern _Bool libcdsb_map_update_uint16_array  (vtype_map* x, vtype_uint16 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_uint16_list   (vtype_map* x, vtype_uint16 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_uint16_map    (vtype_map* x, vtype_uint16 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_uint16_vset   (vtype_map* x, vtype_uint16 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_uint16_boolean(vtype_map* x, vtype_uint16 key,          vtype_bool value);
extern _Bool libcdsb_map_update_uint16_int8   (vtype_map* x, vtype_uint16 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_uint16_int16  (vtype_map* x, vtype_uint16 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_uint16_int32  (vtype_map* x, vtype_uint16 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_uint16_int64  (vtype_map* x, vtype_uint16 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_uint16_uint8  (vtype_map* x, vtype_uint16 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_uint16_uint16 (vtype_map* x, vtype_uint16 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_uint16_uint32 (vtype_map* x, vtype_uint16 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_uint16_uint64 (vtype_map* x, vtype_uint16 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_uint16_float  (vtype_map* x, vtype_uint16 key,         vtype_float value);
extern _Bool libcdsb_map_update_uint16_double (vtype_map* x, vtype_uint16 key,        vtype_double value);
extern _Bool libcdsb_map_update_uint16_ldouble(vtype_map* x, vtype_uint16 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_uint32_pointer(vtype_map* x, vtype_uint32 key, const         void* value);
extern _Bool libcdsb_map_update_uint32_cstring(vtype_map* x, vtype_uint32 key, const         char* value);
extern _Bool libcdsb_map_update_uint32_string (vtype_map* x, vtype_uint32 key, const vtype_string* value);
extern _Bool libcdsb_map_update_uint32_array  (vtype_map* x, vtype_uint32 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_uint32_list   (vtype_map* x, vtype_uint32 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_uint32_map    (vtype_map* x, vtype_uint32 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_uint32_vset   (vtype_map* x, vtype_uint32 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_uint32_boolean(vtype_map* x, vtype_uint32 key,          vtype_bool value);
extern _Bool libcdsb_map_update_uint32_int8   (vtype_map* x, vtype_uint32 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_uint32_int16  (vtype_map* x, vtype_uint32 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_uint32_int32  (vtype_map* x, vtype_uint32 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_uint32_int64  (vtype_map* x, vtype_uint32 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_uint32_uint8  (vtype_map* x, vtype_uint32 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_uint32_uint16 (vtype_map* x, vtype_uint32 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_uint32_uint32 (vtype_map* x, vtype_uint32 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_uint32_uint64 (vtype_map* x, vtype_uint32 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_uint32_float  (vtype_map* x, vtype_uint32 key,         vtype_float value);
extern _Bool libcdsb_map_update_uint32_double (vtype_map* x, vtype_uint32 key,        vtype_double value);
extern _Bool libcdsb_map_update_uint32_ldouble(vtype_map* x, vtype_uint32 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_uint64_pointer(vtype_map* x, vtype_uint64 key, const         void* value);
extern _Bool libcdsb_map_update_uint64_cstring(vtype_map* x, vtype_uint64 key, const         char* value);
extern _Bool libcdsb_map_update_uint64_string (vtype_map* x, vtype_uint64 key, const vtype_string* value);
extern _Bool libcdsb_map_update_uint64_array  (vtype_map* x, vtype_uint64 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_uint64_list   (vtype_map* x, vtype_uint64 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_uint64_map    (vtype_map* x, vtype_uint64 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_uint64_vset   (vtype_map* x, vtype_uint64 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_uint64_boolean(vtype_map* x, vtype_uint64 key,          vtype_bool value);
extern _Bool libcdsb_map_update_uint64_int8   (vtype_map* x, vtype_uint64 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_uint64_int16  (vtype_map* x, vtype_uint64 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_uint64_int32  (vtype_map* x, vtype_uint64 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_uint64_int64  (vtype_map* x, vtype_uint64 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_uint64_uint8  (vtype_map* x, vtype_uint64 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_uint64_uint16 (vtype_map* x, vtype_uint64 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_uint64_uint32 (vtype_map* x, vtype_uint64 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_uint64_uint64 (vtype_map* x, vtype_uint64 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_uint64_float  (vtype_map* x, vtype_uint64 key,         vtype_float value);
extern _Bool libcdsb_map_update_uint64_double (vtype_map* x, vtype_uint64 key,        vtype_double value);
extern _Bool libcdsb_map_update_uint64_ldouble(vtype_map* x, vtype_uint64 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_int8_pointer(vtype_map* x, vtype_int8 key, const         void* value);
extern _Bool libcdsb_map_update_int8_cstring(vtype_map* x, vtype_int8 key, const         char* value);
extern _Bool libcdsb_map_update_int8_string (vtype_map* x, vtype_int8 key, const vtype_string* value);
extern _Bool libcdsb_map_update_int8_array  (vtype_map* x, vtype_int8 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_int8_list   (vtype_map* x, vtype_int8 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_int8_map    (vtype_map* x, vtype_int8 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_int8_vset   (vtype_map* x, vtype_int8 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_int8_boolean(vtype_map* x, vtype_int8 key,          vtype_bool value);
extern _Bool libcdsb_map_update_int8_int8   (vtype_map* x, vtype_int8 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_int8_int16  (vtype_map* x, vtype_int8 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_int8_int32  (vtype_map* x, vtype_int8 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_int8_int64  (vtype_map* x, vtype_int8 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_int8_uint8  (vtype_map* x, vtype_int8 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_int8_uint16 (vtype_map* x, vtype_int8 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_int8_uint32 (vtype_map* x, vtype_int8 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_int8_uint64 (vtype_map* x, vtype_int8 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_int8_float  (vtype_map* x, vtype_int8 key,         vtype_float value);
extern _Bool libcdsb_map_update_int8_double (vtype_map* x, vtype_int8 key,        vtype_double value);
extern _Bool libcdsb_map_update_int8_ldouble(vtype_map* x, vtype_int8 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_int16_pointer(vtype_map* x, vtype_int16 key, const         void* value);
extern _Bool libcdsb_map_update_int16_cstring(vtype_map* x, vtype_int16 key, const         char* value);
extern _Bool libcdsb_map_update_int16_string (vtype_map* x, vtype_int16 key, const vtype_string* value);
extern _Bool libcdsb_map_update_int16_array  (vtype_map* x, vtype_int16 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_int16_list   (vtype_map* x, vtype_int16 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_int16_map    (vtype_map* x, vtype_int16 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_int16_vset   (vtype_map* x, vtype_int16 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_int16_boolean(vtype_map* x, vtype_int16 key,          vtype_bool value);
extern _Bool libcdsb_map_update_int16_int8   (vtype_map* x, vtype_int16 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_int16_int16  (vtype_map* x, vtype_int16 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_int16_int32  (vtype_map* x, vtype_int16 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_int16_int64  (vtype_map* x, vtype_int16 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_int16_uint8  (vtype_map* x, vtype_int16 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_int16_uint16 (vtype_map* x, vtype_int16 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_int16_uint32 (vtype_map* x, vtype_int16 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_int16_uint64 (vtype_map* x, vtype_int16 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_int16_float  (vtype_map* x, vtype_int16 key,         vtype_float value);
extern _Bool libcdsb_map_update_int16_double (vtype_map* x, vtype_int16 key,        vtype_double value);
extern _Bool libcdsb_map_update_int16_ldouble(vtype_map* x, vtype_int16 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_int32_pointer(vtype_map* x, vtype_int32 key, const         void* value);
extern _Bool libcdsb_map_update_int32_cstring(vtype_map* x, vtype_int32 key, const         char* value);
extern _Bool libcdsb_map_update_int32_string (vtype_map* x, vtype_int32 key, const vtype_string* value);
extern _Bool libcdsb_map_update_int32_array  (vtype_map* x, vtype_int32 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_int32_list   (vtype_map* x, vtype_int32 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_int32_map    (vtype_map* x, vtype_int32 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_int32_vset   (vtype_map* x, vtype_int32 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_int32_boolean(vtype_map* x, vtype_int32 key,          vtype_bool value);
extern _Bool libcdsb_map_update_int32_int8   (vtype_map* x, vtype_int32 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_int32_int16  (vtype_map* x, vtype_int32 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_int32_int32  (vtype_map* x, vtype_int32 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_int32_int64  (vtype_map* x, vtype_int32 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_int32_uint8  (vtype_map* x, vtype_int32 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_int32_uint16 (vtype_map* x, vtype_int32 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_int32_uint32 (vtype_map* x, vtype_int32 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_int32_uint64 (vtype_map* x, vtype_int32 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_int32_float  (vtype_map* x, vtype_int32 key,         vtype_float value);
extern _Bool libcdsb_map_update_int32_double (vtype_map* x, vtype_int32 key,        vtype_double value);
extern _Bool libcdsb_map_update_int32_ldouble(vtype_map* x, vtype_int32 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_int64_pointer(vtype_map* x, vtype_int64 key, const         void* value);
extern _Bool libcdsb_map_update_int64_cstring(vtype_map* x, vtype_int64 key, const         char* value);
extern _Bool libcdsb_map_update_int64_string (vtype_map* x, vtype_int64 key, const vtype_string* value);
extern _Bool libcdsb_map_update_int64_array  (vtype_map* x, vtype_int64 key, const  vtype_array* value);
extern _Bool libcdsb_map_update_int64_list   (vtype_map* x, vtype_int64 key, const   vtype_list* value);
extern _Bool libcdsb_map_update_int64_map    (vtype_map* x, vtype_int64 key, const    vtype_map* value);
extern _Bool libcdsb_map_update_int64_vset   (vtype_map* x, vtype_int64 key, const    vtype_set* value);
extern _Bool libcdsb_map_update_int64_boolean(vtype_map* x, vtype_int64 key,          vtype_bool value);
extern _Bool libcdsb_map_update_int64_int8   (vtype_map* x, vtype_int64 key,          vtype_int8 value);
extern _Bool libcdsb_map_update_int64_int16  (vtype_map* x, vtype_int64 key,         vtype_int16 value);
extern _Bool libcdsb_map_update_int64_int32  (vtype_map* x, vtype_int64 key,         vtype_int32 value);
extern _Bool libcdsb_map_update_int64_int64  (vtype_map* x, vtype_int64 key,         vtype_int64 value);
extern _Bool libcdsb_map_update_int64_uint8  (vtype_map* x, vtype_int64 key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_int64_uint16 (vtype_map* x, vtype_int64 key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_int64_uint32 (vtype_map* x, vtype_int64 key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_int64_uint64 (vtype_map* x, vtype_int64 key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_int64_float  (vtype_map* x, vtype_int64 key,         vtype_float value);
extern _Bool libcdsb_map_update_int64_double (vtype_map* x, vtype_int64 key,        vtype_double value);
extern _Bool libcdsb_map_update_int64_ldouble(vtype_map* x, vtype_int64 key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_float_pointer(vtype_map* x, vtype_float key, const         void* value);
extern _Bool libcdsb_map_update_float_cstring(vtype_map* x, vtype_float key, const         char* value);
extern _Bool libcdsb_map_update_float_string (vtype_map* x, vtype_float key, const vtype_string* value);
extern _Bool libcdsb_map_update_float_array  (vtype_map* x, vtype_float key, const  vtype_array* value);
extern _Bool libcdsb_map_update_float_list   (vtype_map* x, vtype_float key, const   vtype_list* value);
extern _Bool libcdsb_map_update_float_map    (vtype_map* x, vtype_float key, const    vtype_map* value);
extern _Bool libcdsb_map_update_float_vset   (vtype_map* x, vtype_float key, const    vtype_set* value);
extern _Bool libcdsb_map_update_float_boolean(vtype_map* x, vtype_float key,          vtype_bool value);
extern _Bool libcdsb_map_update_float_int8   (vtype_map* x, vtype_float key,          vtype_int8 value);
extern _Bool libcdsb_map_update_float_int16  (vtype_map* x, vtype_float key,         vtype_int16 value);
extern _Bool libcdsb_map_update_float_int32  (vtype_map* x, vtype_float key,         vtype_int32 value);
extern _Bool libcdsb_map_update_float_int64  (vtype_map* x, vtype_float key,         vtype_int64 value);
extern _Bool libcdsb_map_update_float_uint8  (vtype_map* x, vtype_float key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_float_uint16 (vtype_map* x, vtype_float key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_float_uint32 (vtype_map* x, vtype_float key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_float_uint64 (vtype_map* x, vtype_float key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_float_float  (vtype_map* x, vtype_float key,         vtype_float value);
extern _Bool libcdsb_map_update_float_double (vtype_map* x, vtype_float key,        vtype_double value);
extern _Bool libcdsb_map_update_float_ldouble(vtype_map* x, vtype_float key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_double_pointer(vtype_map* x, vtype_double key, const         void* value);
extern _Bool libcdsb_map_update_double_cstring(vtype_map* x, vtype_double key, const         char* value);
extern _Bool libcdsb_map_update_double_string (vtype_map* x, vtype_double key, const vtype_string* value);
extern _Bool libcdsb_map_update_double_array  (vtype_map* x, vtype_double key, const  vtype_array* value);
extern _Bool libcdsb_map_update_double_list   (vtype_map* x, vtype_double key, const   vtype_list* value);
extern _Bool libcdsb_map_update_double_map    (vtype_map* x, vtype_double key, const    vtype_map* value);
extern _Bool libcdsb_map_update_double_vset   (vtype_map* x, vtype_double key, const    vtype_set* value);
extern _Bool libcdsb_map_update_double_boolean(vtype_map* x, vtype_double key,          vtype_bool value);
extern _Bool libcdsb_map_update_double_int8   (vtype_map* x, vtype_double key,          vtype_int8 value);
extern _Bool libcdsb_map_update_double_int16  (vtype_map* x, vtype_double key,         vtype_int16 value);
extern _Bool libcdsb_map_update_double_int32  (vtype_map* x, vtype_double key,         vtype_int32 value);
extern _Bool libcdsb_map_update_double_int64  (vtype_map* x, vtype_double key,         vtype_int64 value);
extern _Bool libcdsb_map_update_double_uint8  (vtype_map* x, vtype_double key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_double_uint16 (vtype_map* x, vtype_double key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_double_uint32 (vtype_map* x, vtype_double key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_double_uint64 (vtype_map* x, vtype_double key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_double_float  (vtype_map* x, vtype_double key,         vtype_float value);
extern _Bool libcdsb_map_update_double_double (vtype_map* x, vtype_double key,        vtype_double value);
extern _Bool libcdsb_map_update_double_ldouble(vtype_map* x, vtype_double key,       vtype_ldouble value);

extern _Bool libcdsb_map_update_ldouble_pointer(vtype_map* x, vtype_ldouble key, const         void* value);
extern _Bool libcdsb_map_update_ldouble_cstring(vtype_map* x, vtype_ldouble key, const         char* value);
extern _Bool libcdsb_map_update_ldouble_string (vtype_map* x, vtype_ldouble key, const vtype_string* value);
extern _Bool libcdsb_map_update_ldouble_array  (vtype_map* x, vtype_ldouble key, const  vtype_array* value);
extern _Bool libcdsb_map_update_ldouble_list   (vtype_map* x, vtype_ldouble key, const   vtype_list* value);
extern _Bool libcdsb_map_update_ldouble_map    (vtype_map* x, vtype_ldouble key, const    vtype_map* value);
extern _Bool libcdsb_map_update_ldouble_vset   (vtype_map* x, vtype_ldouble key, const    vtype_set* value);
extern _Bool libcdsb_map_update_ldouble_boolean(vtype_map* x, vtype_ldouble key,          vtype_bool value);
extern _Bool libcdsb_map_update_ldouble_int8   (vtype_map* x, vtype_ldouble key,          vtype_int8 value);
extern _Bool libcdsb_map_update_ldouble_int16  (vtype_map* x, vtype_ldouble key,         vtype_int16 value);
extern _Bool libcdsb_map_update_ldouble_int32  (vtype_map* x, vtype_ldouble key,         vtype_int32 value);
extern _Bool libcdsb_map_update_ldouble_int64  (vtype_map* x, vtype_ldouble key,         vtype_int64 value);
extern _Bool libcdsb_map_update_ldouble_uint8  (vtype_map* x, vtype_ldouble key,         vtype_uint8 value);
extern _Bool libcdsb_map_update_ldouble_uint16 (vtype_map* x, vtype_ldouble key,        vtype_uint16 value);
extern _Bool libcdsb_map_update_ldouble_uint32 (vtype_map* x, vtype_ldouble key,        vtype_uint32 value);
extern _Bool libcdsb_map_update_ldouble_uint64 (vtype_map* x, vtype_ldouble key,        vtype_uint64 value);
extern _Bool libcdsb_map_update_ldouble_float  (vtype_map* x, vtype_ldouble key,         vtype_float value);
extern _Bool libcdsb_map_update_ldouble_double (vtype_map* x, vtype_ldouble key,        vtype_double value);
extern _Bool libcdsb_map_update_ldouble_ldouble(vtype_map* x, vtype_ldouble key,       vtype_ldouble value);

#endif /* LIBCDSB_MAP_H */
