/* This software is licensed by the Apache License 2.0, see LICENSE file */
/*                                       Copyright © 2022 Gregory Lirent */

#include "vnode.h"

#ifndef LIBCDSB_SRC_INTERNAL_RBTREE_H
#define LIBCDSB_SRC_INTERNAL_RBTREE_H

typedef struct libcdsb_rbtree_node {
    struct libcdsb_rbtree_node* left;
    struct libcdsb_rbtree_node* right;
    struct libcdsb_rbtree_node* parent;

    vnode_t value;
    short colored;
} rbnode_t;

extern rbnode_t LIBCDSB_RBTREE_NODE_EMPTY[1];

extern void*     libcdsb_rbtree_node_create(void* value, rbnode_t* parent, int colored, int size);
extern void      libcdsb_rbtree_node_fixup (rbnode_t** root, rbnode_t* node);
extern rbnode_t* libcdsb_rbtree_node_delete(rbnode_t** root, rbnode_t* node);

#define rbnode_empty           ((rbnode_t*)LIBCDSB_RBTREE_NODE_EMPTY)
#define rbnode_create(v, p, c) ((rbnode_t*)libcdsb_rbtree_node_create(v, p, c, sizeof(rbnode_t)))
#define rbnode_fixup           libcdsb_rbtree_node_fixup
#define rbnode_delete          libcdsb_rbtree_node_delete

#define rbnode_is_empty(n) ((n) == rbnode_empty)
#define rbnode_is_root(n)  rbnode_is_empty((n)->parent)

#endif /* LIBCDSB_SRC_INTERNAL_RBTREE_H */
