/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

map_t map_copy(const map_t* s) {

    map_t   x = { .type = s->type };
    stack_t z = { .prev = 0, .value = s->root };
    vtype   t = s->type;

    if (!mnode_is_empty(s->root)) {
        x.root = mnode_create(vnode_duplicate(&s->root->value, t), rbnode_empty, 0);
        stack_push(&z, x.root);

        do {
            mnode_t *p0 = stack_pop(&z);
            mnode_t *p1 = stack_pop(&z);

            if (!mnode_is_empty(p1->left)) {
                p0->left        = mnode_create(vnode_duplicate(&p1->left->key, t), p0, p1->left->colored);
                p0->left->type  = p1->left->type;
                p0->left->value = vnode_duplicate(&p1->left->value, p1->left->type);

                stack_push(&z, p1->left);
                stack_push(&z, p0->left);
            }

            if (!mnode_is_empty(p1->right)) {
                p0->right        = mnode_create(vnode_duplicate(&p1->right->key, t), p0, p1->right->colored);
                p0->right->type  = p1->right->type;
                p0->right->value = vnode_duplicate(&p1->right->value, p1->right->type);

                stack_push(&z, p1->right);
                stack_push(&z, p0->right);
            }

        } while (!is_null(z.value));

    } else x.root = mnode_empty;

    return x;
}


map_t* map_duplicate(const map_t* s) {

    map_t*  x = malloc(sizeof(*x));
    stack_t z = { .prev = 0, .value = s->root };
    vtype   t = x->type = s->type;

    if (!mnode_is_empty(s->root)) {
        x->root = mnode_create(vnode_duplicate(&s->root->value, t), rbnode_empty, 0);
        stack_push(&z, x->root);

        do {
            mnode_t *p0 = stack_pop(&z);
            mnode_t *p1 = stack_pop(&z);

            if (!mnode_is_empty(p1->left)) {
                p0->left        = mnode_create(vnode_duplicate(&p1->left->key, t), p0, p1->left->colored);
                p0->left->type  = p1->left->type;
                p0->left->value = vnode_duplicate(&p1->left->value, p1->left->type);

                stack_push(&z, p1->left);
                stack_push(&z, p0->left);
            }

            if (!mnode_is_empty(p1->right)) {
                p0->right        = mnode_create(vnode_duplicate(&p1->right->key, t), p0, p1->right->colored);
                p0->right->type  = p1->right->type;
                p0->right->value = vnode_duplicate(&p1->right->value, p1->right->type);

                stack_push(&z, p1->right);
                stack_push(&z, p0->right);
            }

        } while (!is_null(z.value));

    } else x->root = mnode_empty;

    return x;
}


void map_copy_init(map_t* x, const map_t* s) {

    stack_t z = { .prev = 0, .value = s->root };
    vtype   t = x->type = s->type;

    if (!mnode_is_empty(s->root)) {
        x->root = mnode_create(vnode_duplicate(&s->root->value, t), rbnode_empty, 0);
        stack_push(&z, x->root);

        do {
            mnode_t *p0 = stack_pop(&z);
            mnode_t *p1 = stack_pop(&z);

            if (!mnode_is_empty(p1->left)) {
                p0->left        = mnode_create(vnode_duplicate(&p1->left->key, t), p0, p1->left->colored);
                p0->left->type  = p1->left->type;
                p0->left->value = vnode_duplicate(&p1->left->value, p1->left->type);

                stack_push(&z, p1->left);
                stack_push(&z, p0->left);
            }

            if (!mnode_is_empty(p1->right)) {
                p0->right        = mnode_create(vnode_duplicate(&p1->right->key, t), p0, p1->right->colored);
                p0->right->type  = p1->right->type;
                p0->right->value = vnode_duplicate(&p1->right->value, p1->right->type);

                stack_push(&z, p1->right);
                stack_push(&z, p0->right);
            }

        } while (!is_null(z.value));

    } else x->root = mnode_empty;
}
