/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

size_t libcdsb_string_replace_cstring_cstring(str_t* x, const char* a, const char* d, size_t maxn) {
    char *p, *t, *r;
    size_t c, n, an, dn;

    if (a == d) return string_count(x, a);

    if (is_null(x->buffer) || is_null(a) || !*x->buffer || !*a) {
        return 0;
    }

    an = strlen(a);
    dn = (!is_null(d)) ? strlen(d) : 0;
    n  = strlen(x->buffer);
    p  = x->buffer;
    c  = 0;
    t  = 0;
    r  = 0;

    if (x->buffer == a) {
        x->buffer = realloc(x->buffer, dn + 1);
        memcpy(x->buffer, d, dn);
        x->buffer[dn] = 0;

        return 1;
    }

    if (x->buffer < a && a < x->buffer + n) {
        a = r = memndup(a, an);
    }

    if (x->buffer <= d && d < x->buffer + n) {
        d = t = memndup(d, dn);
    }

    while (maxn-- && !is_null(p = strstr(p, a))) {
        libcdsb_string_replace(x, p, an, d, dn);
        p += an;
        ++c;
    }

    free(r);
    free(t);
    return c;
}


size_t libcdsb_string_replace_cstring_char(str_t* x, const char* a, int dc, size_t maxn) {
    char *p, *t;
    char d[4];
    size_t c, n, an, dn;

    if (is_null(x->buffer) || is_null(a) || !*x->buffer || !*a) {
        return 0;
    }

    an = strlen(a);
    p  = tochar_unicode(d, dc);
    dn = (!is_null(p)) ? p - d : 0;
    n  = strlen(x->buffer);
    p  = x->buffer;
    c  = 0;
    t  = 0;

    if (x->buffer == a) {
        x->buffer = realloc(x->buffer, dn + 1);
        memcpy(x->buffer, d, dn);
        x->buffer[dn] = 0;

        return 1;
    } else if (x->buffer < a && a < x->buffer + n) {
        a = t = memndup(a, an);
    }

    while (maxn-- && !is_null(p = strstr(p, a))) {
        libcdsb_string_replace(x, p, an, d, dn);
        p += an;
        ++c;
    }

    free(t);
    return c;
}


size_t libcdsb_string_replace_char_cstring(str_t* x, int ac, const char* d, size_t maxn) {
    char *p, *t;
    char a[4];
    size_t c, n, an, dn;

    p = tochar_unicode(a, ac);

    if (is_null(x->buffer) || is_null(p) || !*x->buffer || !*p) {
        return 0;
    }

    an = p - a;
    dn = (!is_null(d)) ? strlen(d) : 0;
    n  = strlen(x->buffer);
    p  = x->buffer;
    c  = 0;
    t  = 0;

    if (x->buffer <= d && d < x->buffer + n) {
        d = t = memndup(d, dn);
    }

    while (maxn-- && !is_null(p = strstr(p, a))) {
        libcdsb_string_replace(x, p, an, d, dn);
        p += an;
        ++c;
    }

    free(t);
    return c;
}


size_t libcdsb_string_replace_char_char(str_t* x, int ac, int dc, size_t maxn) {
    char* p;
    char a[4];
    char d[4];
    size_t c, an, dn;

    p = tochar_unicode(a, ac);

    if (is_null(x->buffer) || is_null(p) || !*x->buffer || !*p) {
        return 0;
    }

    an = p - a;
    p  = tochar_unicode(d, dc);
    dn = (!is_null(p)) ? p - d : 0;
    p  = x->buffer;
    c  = 0;

    while (maxn-- && !is_null(p = strstr(p, a))) {
        libcdsb_string_replace(x, p, an, d, dn);
        p += an;
        ++c;
    }

    return c;
}



size_t libcdsb_string_replace_r_cstring_cstring(str_t* x, const char*restrict a, const char*restrict d, size_t maxn) {
    char *restrict p;
    size_t c, an, dn;

    if (is_null(x->buffer) || is_null(a) || !*x->buffer || !*a) {
        return 0;
    }

    an = strlen(a);
    dn = (!is_null(d)) ? strlen(d) : 0;
    p  = x->buffer;
    c  = 0;

    while (maxn-- && !is_null(p = strstr(p, a))) {
        libcdsb_string_replace(x, p, an, d, dn);
        p += an;
        ++c;
    }

    return c;
}


size_t libcdsb_string_replace_r_cstring_char(str_t* x, const char*restrict a, int dc, size_t maxn) {
    char *restrict p;
    char d[4];
    size_t c, an, dn;

    if (is_null(x->buffer) || is_null(a) || !*x->buffer || !*a) {
        return 0;
    }

    an = strlen(a);
    p  = tochar_unicode(d, dc);
    dn = (!is_null(p)) ? p - d : 0;
    p  = x->buffer;
    c  = 0;

    while (maxn-- && !is_null(p = strstr(p, a))) {
        libcdsb_string_replace(x, p, an, d, dn);
        p += an;
        ++c;
    }

    return c;
}


size_t libcdsb_string_replace_r_char_cstring(str_t* x, int ac, const char*restrict d, size_t maxn) {
    char *restrict p;
    char a[4];
    size_t c, an, dn;

    p = tochar_unicode(a, ac);

    if (is_null(x->buffer) || is_null(p) || !*x->buffer || !*p) {
        return 0;
    }

    an = p - a;
    dn = (!is_null(d)) ? strlen(d) : 0;
    p  = x->buffer;
    c  = 0;

    while (maxn-- && !is_null(p = strstr(p, a))) {
        libcdsb_string_replace(x, p, an, d, dn);
        p += an;
        ++c;
    }

    return c;
}
